Package: vcd
Version: 1.4-13
Title: Visualizing Categorical Data
Authors@R: c(person(given = "David", family = "Meyer", role = c("aut", "cre"),
                    email = "David.Meyer@R-project.org",
		    comment = c(ORCID = "0000-0002-5196-3048")),	    
             person(given = "Achim", family = "Zeileis", role = "aut",
	            email = "Achim.Zeileis@R-project.org",
                    comment = c(ORCID = "0000-0003-0918-3766")),
             person(given = "Kurt", family = "Hornik", role = "aut",
	            email = "Kurt.Hornik@R-project.org",
                    comment = c(ORCID = "0000-0003-4198-9911")),
             person(given = "Florian", family = "Gerber", role = "ctb"),
             person(given = "Michael", family = "Friendly", role = "aut",
	            email = "friendly@yorku.ca",
		    comment = c(ORCID = "0000-0002-3237-0941")))
Description: Visualization techniques, data sets, summary and inference
        procedures aimed particularly at categorical data. Special
        emphasis is given to highly extensible grid graphics. The
        package was package was originally inspired by the book 
	"Visualizing Categorical Data" by Michael Friendly and is 
	now the main support package for a new book, 
	"Discrete Data Analysis with R" by Michael Friendly and 
	David Meyer (2015).
LazyLoad: yes
LazyData: yes
Depends: R (>= 2.4.0), grid
Suggests: KernSmooth, mvtnorm, kernlab, HSAUR3, coin
Imports: stats, utils, MASS, grDevices, colorspace, lmtest
License: GPL-2
NeedsCompilation: no
Packaged: 2024-09-16 09:01:34 UTC; meyer
Author: David Meyer [aut, cre] (<https://orcid.org/0000-0002-5196-3048>),
  Achim Zeileis [aut] (<https://orcid.org/0000-0003-0918-3766>),
  Kurt Hornik [aut] (<https://orcid.org/0000-0003-4198-9911>),
  Florian Gerber [ctb],
  Michael Friendly [aut] (<https://orcid.org/0000-0002-3237-0941>)
Maintainer: David Meyer <David.Meyer@R-project.org>
Repository: CRAN
Date/Publication: 2024-09-16 10:13:35 UTC
Built: R 4.6.0; ; 2026-04-25 23:39:27 UTC; unix
